"""FIXME

$Id: Examen.py 7 2008-04-18 14:46:13Z gael.le-mignot $
"""

from AccessControl import ClassSecurityInfo

from AccessControl import Unauthorized, getSecurityManager

from Products.Archetypes.public import *
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.content.folder import ATFolder, ATFolderSchema

from Products.OnlineExamen.config import PROJECT_NAME 

from BTrees.OOBTree import OOBTree

import csv, time
import DateTime
from cStringIO import StringIO

import logging
log = logging.getLogger(PROJECT_NAME)

schema = ATFolderSchema.copy()
finalizeATCTSchema(schema, folderish = True)


class Examen(ATFolder):
    """An examen containing questions"""

    ## Type, icon and description
    meta_type = portal_type = 'Examen'
    archetype_name = 'Examen'
    content_icon = 'examen_icon.png'
    typeDescription = 'An examen containing questions'
    typeDescMsgId = 'examen_description'

    ## Views
    default_view = 'examen_view'
    immediate_view = default_view
    suppl_views = ()

    ## Actions
    actions = ATFolder.actions + ( {'id' : 'show_answers',
                                    'name' : 'Show answers',
                                    'action' : 'string:${object_url}/show_answers',
                                    'permissions' : "Manage portal",  
                                    },
                                   {'id'          : 'export_csv_marks',
                                    'name'        : 'CSV export of marks',
                                    'action'      : 'string:${object_url}/export_csv_grades',
                                    'permissions' : "Manage portal",
                                    },
                                   )

    ## Various settings
    _at_rename_after_creation = True
    global_allow = True
    allow_discussion = False
    filter_content_types = True
    allowed_content_types = ('Question', )

    ## Security shortcuts
    security = ClassSecurityInfo()

    schema = schema

    def _getSubObject(self, *args, **kwargs):
        """
        Ensure we are either manager or effective date is in the future
        # FIXME
        """
        user = getSecurityManager().getUser()
        log.info("Checking access for user %s" % user.getId())
        if not user.has_role("Manager"):
            raise Unauthorized
        return ATFolder.getSubObject(self, *args, **kwargs)

    security.declareProtected("View", "getAnswersFor")
    def getAnswersFor(self, username = None):
        """
        Get answers for given or current user
        If user is given, must be manager
        """
        user = getSecurityManager().getUser()
        if user.has_role("Anonymous"):
            return {}

        if username is None:
            username = user.getId()
        else:
            if not user.has_role("Manager"):
                raise Unauthorized

        mapping = self.getAnswersMapping()
        return mapping.get(username, {})

    security.declarePrivate("setAnswersFor")
    def setAnswersFor(self, username, answers = {}):
        """
        Set answers for given user (well, update)
        """
        mapping = self.getAnswersMapping()
        values = mapping.get(username, {})
        values.update(answers)
        mapping[username] = values


    security.declarePrivate("getAnswersMapping")
    def getAnswersMapping(self):
        """
        Get the answers mapping
        """
        answers = getattr(self, "_answers", None)
        if answers is None:
            answers = OOBTree()
            setattr(self, "_answers", answers)
        return answers    

    security.declareProtected("View", "getQuestions")
    def getQuestions(self):
        """
        Get all questions
        """
        return self.getFolderContents(contentFilter={ "portal_type": ("Question",),
                                                      }, full_objects = True)

    security.declareProtected("View", "processAnswers")
    def processAnswers(self):
        """
        Process answers from form
        """
        user = getSecurityManager().getUser()
        if user.has_role("Anonymous"):
            return self.translate("You must be identified to answer", domain='examen')
        username = user.getId()

        answers = {}
        form = self.REQUEST.form
        if form.has_key("submitted"):
            for key in form.keys():
                if key.startswith("question-"):
                    answers[key] = form[key]

            answers["time"] = DateTime.DateTime()
            self.setAnswersFor(username, answers)
            return self.translate("Answers saved", domain='examen')

    security.declareProtected("Manage portal", "getAnswerers")
    def getAnswerers(self):
        """
        Get the list of all people who answered
        """
        keys = list(self.getAnswersMapping().keys())
        keys.sort()
        return keys

    security.declareProtected("Manage portal", "processMarks")
    def processMarks(self):
        """
        Process marks from form
        """
        answers = {}
        form = self.REQUEST.form
        if form.has_key("submitted"):
            answerers = self.getAnswerers()
            current = form.get("current", 0)
            username = answerers[current]
            for key in form.keys():
                if key.startswith("comment-") or key.startswith("mark-"):
                    answers[key] = form[key]

            self.setAnswersFor(username, answers)
            return self.translate("Comments saved", domain='examen')

    security.declarePrivate("export_csv")
    def export_csv(self, name, data):
        """
        Do a CSV export from a Python list
        """
        buffer = StringIO()
        writer = csv.writer(buffer, quoting = csv.QUOTE_ALL)
        for row in data:
            writer.writerow(row)
        value = buffer.getvalue()
        value = unicode(value, "utf-8").encode("iso-8859-1", "replace")
        self.REQUEST.RESPONSE.setHeader('Content-Type', 'text/csv')
        self.REQUEST.RESPONSE.setHeader('Content-Disposition', 'attachment;filename=%s-%s.csv' % (name, time.strftime("%Y%m%d-%H%M")))
        return value

    security.declareProtected("Manage portal", "export_csv_grades")
    def export_csv_grades(self):
        """
        Export grades in CSV
        """
        questions = self.getQuestions()
        qnames = [ q.title_or_id() for q in questions ]
        data = [ [ "Login", "Date", ] + qnames + [ "Total" ] ]
        for answerer in self.getAnswerers():
            answers = self.getAnswersFor(answerer)
            values = [ answers.get("mark-%s" % q.getId(), 0) for q in questions ]
            row = [ answerer, answers["time"].strftime("%Y-%m-%d %H:%M:%S") ] + values + [ sum(values) ]
            data.append(row)

        return self.export_csv(self.getId(), data)
                   


registerType(Examen)
